@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 #
 # Wrapper script for emscripten build.
 #
@@ -10,7 +10,7 @@ make_wrapper_script() {
     toolname="$1"
     emtoolname="$2"
     path="$EM_CACHE/bin/asmjs-local-emscripten-$toolname"
-    (echo "#!/bin/bash"
+    (echo "#!/usr/bin/env bash"
      echo "exec $emtoolname \"\$@\"") > "$path"
     chmod a+rx "$path"
 }
@@ -22,7 +22,7 @@ export EM_FROZEN_CACHE=
 export PKG_CONFIG_PATH=$EM_CACHE/pkg
 export PATH="$EM_CACHE/bin:$PATH"
 export CFLAGS="-fexceptions -sSUPPORT_LONGJMP=emscripten"
-export LDFLAGS="-flto -sASYNCIFY -sENVIRONMENT=web -lidbfs.js -Wl,-u,ntohs $CFLAGS"
+export LDFLAGS="-flto -sASYNCIFY -sFORCE_FILESYSTEM -sEXPORTED_FUNCTIONS=_main -sENVIRONMENT=web -lidbfs.js -Wl,-u,ntohs $CFLAGS"
 
 # The first time creating the cache directory, we make a copy of the
 # one already installed on the system, to avoid having to rebuild all
@@ -55,7 +55,7 @@ make_wrapper_script ld     emcc
 make_wrapper_script nm     emnm
 make_wrapper_script ranlib emranlib
 
-if [ ! -e configure ]; then
+if [ ! -e configure ] || [ ! -e Makefile ]; then
     ./autogen.sh --host=asmjs-local-emscripten
 fi
-make
+exec make "$@"
