@@ -2,12 +2,11 @@
 // Copyright(C) 1984-2000 David L. Clark
 // Copyright(C) 2001-2005 Simon Howard
 //
-// This program is free software; you can redistribute it and/or modify it
-// under the terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2 of the License, or (at your
-// option) any later version. This program is distributed in the hope that
-// it will be useful, but WITHOUT ANY WARRANTY; without even the implied
-// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// You can redistribute and/or modify this program under the terms of the
+// GNU General Public License version 2 as published by the Free Software
+// Foundation, or any later version. This program is distributed WITHOUT
+// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.
 //
 //
 // PC Speaker Interface
@@ -16,6 +15,8 @@
 #ifndef __SDLSOUND_H__
 #define __SDLSOUND_H__
 
+#include "std.h"
+
 void Speaker_Init(void);
 void Speaker_Off(void);
 
@@ -30,6 +31,6 @@ void Speaker_Off(void);
 
 void Speaker_Output(unsigned short count);
 
-extern int snd_tinnyfilter;
+extern bool snd_tinnyfilter;
 
 #endif
