@@ -28,6 +28,12 @@
 
 #define INPUT_BUFFER_LEN 32
 
+void Vid_ControllerButtonDown(SDL_ControllerButtonEvent *event);
+void Vid_ControllerButtonUp(SDL_ControllerButtonEvent *event);
+void Vid_ControllerAdded(SDL_ControllerDeviceEvent *event);
+void Vid_ControllerRemoved(SDL_ControllerDeviceEvent *event);
+void Vid_ControllerInit(void);
+
 struct palette {
 	char name[13]; // Up to 12 characters will display correctly on the menu
 	SDL_Color color[4];
@@ -67,6 +73,9 @@ static const struct palette video_palettes[] = {
 	{"Atari",
 		{{0x00, 0x00, 0x00}, {0x00, 0x77, 0xff},
 		 {0xff, 0x00, 0x00}, {0xff, 0xff, 0xff}}},
+	{"Muted",
+		{{0x00, 0x00, 0x00}, {0x78, 0xc3, 0xd6},
+		 {0xc5, 0x51, 0xc5}, {0xc7, 0xc7, 0xc7}}},
 };
 
 extern bool isNetworkGame(void);
@@ -86,7 +95,12 @@ int keybindings[NUM_KEYS] = {
 	SDL_SCANCODE_S,       // KEY_SOUND
 };
 
+// Button that each finger is currently pressing. We assume the user does
+// not have more than 16 fingers.
+static const struct touch_button *pressed_buttons[16];
+
 bool vid_fullscreen = false;
+static bool touch_area_enabled = false;
 static int ctrlbreak = 0;
 static bool initted = false;
 static SDL_Window *window = NULL;
@@ -153,9 +167,18 @@ static SDL_Surface *SurfaceFromSopsym(sopsym_t *sym)
 	return surface;
 }
 
+static int TouchAreaHeight(void)
+{
+	if (touch_area_enabled) {
+		return TOUCH_AREA_HEIGHT;
+	}
+
+	return 0;
+}
+
 void Vid_Update(void)
 {
-	static SDL_Rect blit_rect = { 0, 0, SCR_WDTH, SCR_HGHT };
+	SDL_Rect blit_rect = {0, 0, SCR_WDTH, SCR_HGHT + TouchAreaHeight()};
 
 	if (!initted) {
 		Vid_Init();
@@ -163,6 +186,12 @@ void Vid_Update(void)
 
 	SDL_UnlockSurface(screenbuf);
 
+	if (touch_area_enabled) {
+		vid_vram += SCR_HGHT * vid_pitch;
+		Vid_DrawTouchControls();
+		vid_vram -= SCR_HGHT * vid_pitch;
+	}
+
 	// Blit from the paletted 8-bit screen buffer to the intermediate
 	// 32-bit RGBA buffer that we can load into the texture.
 	SDL_LowerBlit(screenbuf, &blit_rect, argbbuffer, &blit_rect);
@@ -231,6 +260,7 @@ static void SetIcon(void)
 static void LimitTextureSize(int *w_upscale, int *h_upscale)
 {
 	SDL_RendererInfo rinfo;
+	int height = SCR_HGHT + TouchAreaHeight();
 	int orig_w, orig_h;
 
 	orig_w = *w_upscale;
@@ -245,7 +275,7 @@ static void LimitTextureSize(int *w_upscale, int *h_upscale)
 	while (*w_upscale * SCR_WDTH > rinfo.max_texture_width) {
 		--*w_upscale;
 	}
-	while (*h_upscale * SCR_HGHT > rinfo.max_texture_height) {
+	while (*h_upscale * height > rinfo.max_texture_height) {
 		--*h_upscale;
 	}
 
@@ -262,14 +292,14 @@ static void LimitTextureSize(int *w_upscale, int *h_upscale)
 	// depending on the hardware there may be performance problems with very
 	// huge textures, so the user can use this to reduce the maximum texture
 	// size if desired.
-	if (max_scaling_buffer_pixels < SCR_WDTH * SCR_HGHT) {
+	if (max_scaling_buffer_pixels < SCR_WDTH * height) {
 		ErrorExit("CreateUpscaledTexture: max_scaling_buffer_"
 		          "pixels too small to create a texture buffer:"
 		          " %d < %d", max_scaling_buffer_pixels,
-		          SCR_WDTH * SCR_HGHT);
+		          SCR_WDTH * height);
 	}
 
-	while (*w_upscale * *h_upscale * SCR_WDTH * SCR_HGHT
+	while (*w_upscale * *h_upscale * SCR_WDTH * height
 	       > max_scaling_buffer_pixels) {
 		if (*w_upscale > *h_upscale) {
 			--*w_upscale;
@@ -281,7 +311,7 @@ static void LimitTextureSize(int *w_upscale, int *h_upscale)
 	if (*w_upscale != orig_w || *h_upscale != orig_h) {
 		printf("CreateUpscaledTexture: Limited texture size to %dx%d "
 		       "(max %d pixels, max texture size %dx%d)",
-		       *w_upscale * SCR_WDTH, *h_upscale * SCR_HGHT,
+		       *w_upscale * SCR_WDTH, *h_upscale * height,
 		       max_scaling_buffer_pixels, rinfo.max_texture_width,
 		       rinfo.max_texture_height);
 	}
@@ -290,6 +320,7 @@ static void LimitTextureSize(int *w_upscale, int *h_upscale)
 static void CreateUpscaledTexture(int force)
 {
 	static int h_upscale_old, w_upscale_old;
+	int height = SCR_HGHT + TouchAreaHeight();
 	int w, h;
 	int h_upscale, w_upscale;
 	SDL_Texture *new_texture, *old_texture;
@@ -304,27 +335,20 @@ static void CreateUpscaledTexture(int force)
 	// When the screen or window dimensions do not match the aspect ratio
 	// of the texture, the rendered area is scaled down to fit. Calculate
 	// the actual dimensions of the rendered area.
-	if (w * SCR_HGHT < h * SCR_WDTH) {
+	if (w * height < h * SCR_WDTH) {
 		// Tall window.
-		h = w * SCR_HGHT / SCR_WDTH;
+		h = w * height / SCR_WDTH;
 	} else {
 		// Wide window.
-		w = h * SCR_WDTH / SCR_HGHT;
+		w = h * SCR_WDTH / height;
 	}
 
 	// Pick texture size the next integer multiple of the screen dimensions.
 	// If one screen dimension matches an integer multiple of the original
 	// resolution, there is no need to overscale in this direction.
-	w_upscale = (w + SCR_WDTH - 1) / SCR_WDTH;
-	h_upscale = (h + SCR_HGHT - 1) / SCR_HGHT;
-
 	// Minimum texture dimensions of 320x200.
-	if (w_upscale < 1) {
-		w_upscale = 1;
-	}
-	if (h_upscale < 1) {
-		h_upscale = 1;
-	}
+	w_upscale = clamp_min(1, (w + SCR_WDTH - 1) / SCR_WDTH);
+	h_upscale = clamp_min(1, (h + height - 1) / height);
 
 	LimitTextureSize(&w_upscale, &h_upscale);
 
@@ -343,7 +367,7 @@ static void CreateUpscaledTexture(int force)
 
 	new_texture = SDL_CreateTexture(
 		renderer, pixel_format, SDL_TEXTUREACCESS_TARGET,
-		w_upscale*SCR_WDTH, h_upscale*SCR_HGHT);
+		w_upscale*SCR_WDTH, h_upscale*height);
 
 	old_texture = texture_upscaled;
 	texture_upscaled = new_texture;
@@ -353,20 +377,13 @@ static void CreateUpscaledTexture(int force)
 	}
 }
 
-static void Vid_UnsetMode(void)
-{
-	SDL_DestroyRenderer(renderer);
-	SDL_DestroyWindow(window);
-	SDL_QuitSubSystem(SDL_INIT_VIDEO);
-}
-
 static void GetWindowSize(int *w, int *h)
 {
 	SDL_DisplayMode mode;
 	int factor;
 
 	*w = SCR_WDTH;
-	*h = SCR_HGHT;
+	*h = SCR_HGHT + TouchAreaHeight();
 
 	if (SDL_GetDesktopDisplayMode(0, &mode) != 0) {
 		return;
@@ -401,8 +418,20 @@ static void Vid_SetMode(void)
 		flags |= SDL_WINDOW_FULLSCREEN_DESKTOP;
 	}
 
-	window = SDL_CreateWindow(PACKAGE_STRING, SDL_WINDOWPOS_CENTERED,
-	                          SDL_WINDOWPOS_CENTERED, w, h, flags);
+	if (window == NULL) {
+		window = SDL_CreateWindow(PACKAGE_STRING, SDL_WINDOWPOS_CENTERED,
+		                          SDL_WINDOWPOS_CENTERED, w, h, flags);
+	} else {
+		SDL_SetWindowFullscreen(window, flags);
+#ifndef __EMSCRIPTEN__
+		// SDL's Emscripten backend automatically resizes windows if
+		// they are created with SDL_WINDOW_RESIZABLE, but this gets
+		// overridden if we call SDL_SetWindowSize() without actually
+		// changing the canvas size, leading to a stretched out image
+		// in the wrong aspect ratio.
+		SDL_SetWindowSize(window, w, h);
+#endif
+	}
 
 	if (window == NULL) {
 		ErrorExit("Failed to open SDL window: %s", SDL_GetError());
@@ -419,13 +448,16 @@ static void Vid_SetMode(void)
 	SetIcon();
 	SDL_ShowCursor(0);
 
-	renderer_flags = SDL_RENDERER_PRESENTVSYNC;
-	renderer = SDL_CreateRenderer(window, -1, renderer_flags);
+	if (renderer == NULL) {
+		renderer_flags = SDL_RENDERER_PRESENTVSYNC;
+		renderer = SDL_CreateRenderer(window, -1, renderer_flags);
+	}
 
 	// Important: Set the "logical size" of the rendering context. At the
 	// same time this also defines the aspect ratio that is preserved while
 	// scaling and stretching the texture into the window.
-	SDL_RenderSetLogicalSize(renderer, SCR_WDTH, SCR_HGHT);
+	SDL_RenderSetLogicalSize(renderer, SCR_WDTH,
+	                         SCR_HGHT + TouchAreaHeight());
 
 	// Blank out the full screen area in case there is any junk in
 	// the borders that won't otherwise be overwritten.
@@ -444,8 +476,10 @@ static void Vid_SetMode(void)
 		SDL_PixelFormatEnumToMasks(
 			pixel_format, &bpp, &rmask, &gmask,
 			&bmask, &amask);
+		// The texture we create is larger than the screen in height,
+		// so that we can display the touch controls if enabled.
 		argbbuffer = SDL_CreateRGBSurface(
-			0, SCR_WDTH, SCR_HGHT, bpp,
+			0, SCR_WDTH, SCR_HGHT + TOUCH_AREA_HEIGHT, bpp,
 			rmask, gmask, bmask, amask);
 		SDL_FillRect(argbbuffer, NULL, 0);
 	}
@@ -464,7 +498,7 @@ static void Vid_SetMode(void)
 	// texture's content is going to change frequently.
 	texture = SDL_CreateTexture(renderer, pixel_format,
 	                            SDL_TEXTUREACCESS_STREAMING,
-	                            SCR_WDTH, SCR_HGHT);
+	                            SCR_WDTH, SCR_HGHT + TouchAreaHeight());
 
 	// Initially create the upscaled texture for rendering to screen
 	CreateUpscaledTexture(1);
@@ -476,9 +510,12 @@ void Vid_Shutdown(void)
 		return;
 	}
 
-	Vid_UnsetMode();
-
+	SDL_DestroyRenderer(renderer);
+	renderer = NULL;
+	SDL_DestroyWindow(window);
+	window = NULL;
 	SDL_FreeSurface(screenbuf);
+	SDL_QuitSubSystem(SDL_INIT_VIDEO);
 
 	initted = false;
 }
@@ -490,11 +527,12 @@ void Vid_Init(void)
 	}
 
 	fflush(stdout);
+	touch_area_enabled = SDL_GetNumTouchDevices() > 0;
 
 	Vid_SetMode();
 
-	screenbuf = SDL_CreateRGBSurface(0, SCR_WDTH, SCR_HGHT, 8,
-	                                 0, 0, 0, 0);
+	screenbuf = SDL_CreateRGBSurface(
+		0, SCR_WDTH, SCR_HGHT + TOUCH_AREA_HEIGHT, 8, 0, 0, 0, 0);
 	vid_vram = screenbuf->pixels;
 	vid_pitch = screenbuf->pitch;
 	SDL_SetPaletteColors(screenbuf->format->palette,
@@ -505,6 +543,8 @@ void Vid_Init(void)
 
 	atexit(Vid_Shutdown);
 
+	Vid_ControllerInit();
+
 	SDL_LockSurface(screenbuf);
 }
 
@@ -514,7 +554,6 @@ void Vid_Reset(void)
 		return;
 	}
 
-	Vid_UnsetMode();
 	Vid_SetMode();
 
 	// need to redraw buffer to screen
@@ -657,7 +696,7 @@ static void KeyDown(SDL_KeyboardEvent *event)
 
 	translated = TranslateScancode(event->keysym.scancode);
 	if (translated != KEY_UNKNOWN) {
-		keysdown[translated] |= 3;
+		keysdown[translated] |= KEYDOWN_KEYBOARD | KEYDOWN_WAS_PRESSED;
 	}
 }
 
@@ -665,7 +704,110 @@ static void KeyUp(SDL_KeyboardEvent *event)
 {
 	enum gamekey translated = TranslateScancode(event->keysym.scancode);
 	if (translated != KEY_UNKNOWN) {
-		keysdown[translated] &= ~1;
+		keysdown[translated] &= ~KEYDOWN_KEYBOARD;
+	}
+}
+
+static bool TouchCoordToPixel(float x, float y, int *px, int *py)
+{
+	int height = SCR_HGHT + TouchAreaHeight();
+
+	*px = (int) (x * SCR_WDTH);
+	*py = (int) (y * height);
+
+	return *px >= 0 && *px < SCR_WDTH && *py >= 0 && *py < height;
+}
+
+static bool ValidFinger(SDL_TouchFingerEvent *ev)
+{
+	// TODO: Support more than two touch devices:
+	return ev->touchId == SDL_GetTouchDevice(0)
+	    && ev->fingerId < arrlen(pressed_buttons);
+}
+
+static const struct touch_button *GetTouchButton(SDL_TouchFingerEvent *ev)
+{
+	int x, y;
+
+	if (!TouchCoordToPixel(ev->x, ev->y, &x, &y) || y < SCR_HGHT) {
+		return NULL;
+	}
+	return Vid_GetTouchButton(x, y - SCR_HGHT);
+}
+
+static void FingerDown(SDL_TouchFingerEvent *ev)
+{
+	const struct touch_button *b;
+
+	if (!ValidFinger(ev)) {
+		return;
+	}
+
+	b = GetTouchButton(ev);
+	pressed_buttons[ev->fingerId] = b;
+	Vid_TouchButtonPress(b, true);
+	if (b == NULL) {
+		return;
+	}
+
+	switch (b->type) {
+	case TOUCH_BUTTON_GAME_KEY:
+		keysdown[b->param] |= KEYDOWN_TOUCH | KEYDOWN_WAS_PRESSED;
+		break;
+
+	default:
+		break;
+	}
+}
+
+static void FingerUp(SDL_TouchFingerEvent *ev)
+{
+	SDL_Keysym fake = {0};
+	const struct touch_button *b;
+
+	if (!ValidFinger(ev)) {
+		return;
+	}
+	b = pressed_buttons[ev->fingerId];
+	if (b == NULL) {
+		return;
+	}
+	switch (b->type) {
+	case TOUCH_BUTTON_GAME_KEY:
+		keysdown[b->param] &= ~KEYDOWN_TOUCH;
+		break;
+
+	case TOUCH_BUTTON_CLOSE:
+		touch_area_enabled = false;
+		Vid_Reset();
+		break;
+
+	case TOUCH_BUTTON_KEYPRESS:
+		fake.sym = b->param;
+		fake.scancode = SDL_SCANCODE_UNKNOWN;
+		InputBufferPush(fake);
+		break;
+
+	default:
+		break;
+	}
+	pressed_buttons[ev->fingerId] = NULL;
+	Vid_TouchButtonPress(b, false);
+}
+
+static void FingerMove(SDL_TouchFingerEvent *ev)
+{
+	const struct touch_button *b, *old_button;
+
+	if (!ValidFinger(ev)) {
+		return;
+	}
+
+	old_button = pressed_buttons[ev->fingerId];
+	b = GetTouchButton(ev);
+	if (b != old_button) {
+		FingerUp(ev);
+		FingerDown(ev);
 	}
 }
 
@@ -711,6 +853,40 @@ static void GetEvents(void)
 				need_redraw = true;
 				break;
 			}
+			break;
+
+		case SDL_FINGERDOWN:
+			if (!touch_area_enabled) {
+				touch_area_enabled = true;
+				Vid_Reset();
+			} else {
+				FingerDown(&event.tfinger);
+			}
+			break;
+
+		case SDL_FINGERUP:
+			FingerUp(&event.tfinger);
+			break;
+
+		case SDL_FINGERMOTION:
+			FingerMove(&event.tfinger);
+			break;
+
+		case SDL_CONTROLLERBUTTONDOWN:
+			Vid_ControllerButtonDown(&event.cbutton);
+			break;
+
+		case SDL_CONTROLLERBUTTONUP:
+			Vid_ControllerButtonUp(&event.cbutton);
+			break;
+
+		case SDL_CONTROLLERDEVICEADDED:
+			Vid_ControllerAdded(&event.cdevice);
+			break;
+
+		case SDL_CONTROLLERDEVICEREMOVED:
+			Vid_ControllerRemoved(&event.cdevice);
+			break;
 		}
 	}
 
