@@ -26,6 +26,7 @@
 #include "sw.h"
 #include "swconf.h"
 #include "swend.h"
+#include "swmenu.h"
 #include "swsound.h"
 #include "swtext.h"
 #include "swtitle.h"
@@ -81,6 +82,15 @@ static const struct conf_option confoptions[] = {
 	{"key_home",       CONF_KEY, {&keybindings[KEY_HOME]}},
 	{"key_missile",    CONF_KEY, {&keybindings[KEY_MISSILE]}},
 	{"key_starburst",  CONF_KEY, {&keybindings[KEY_STARBURST]}},
+
+	{"controller_accelerate", CONF_INT, {&controller_bindings[KEY_ACCEL]}},
+	{"controller_decelerate", CONF_INT, {&controller_bindings[KEY_DECEL]}},
+	{"controller_pullup",     CONF_INT, {&controller_bindings[KEY_PULLUP]}},
+	{"controller_pulldown",   CONF_INT, {&controller_bindings[KEY_PULLDOWN]}},
+	{"controller_flip",       CONF_INT, {&controller_bindings[KEY_FLIP]}},
+	{"controller_fire",       CONF_INT, {&controller_bindings[KEY_FIRE]}},
+	{"controller_dropbomb",   CONF_INT, {&controller_bindings[KEY_BOMB]}},
+	{"controller_home",       CONF_INT, {&controller_bindings[KEY_HOME]}},
 };
 
 static void Chomp(char *s)
@@ -91,7 +101,7 @@ static void Chomp(char *s)
 	}
 }
 
-static const struct conf_option *ConfOptionByName(const char *name)
+const struct conf_option *ConfOptionByName(const char *name)
 {
 	int i;
 
@@ -236,7 +246,7 @@ void swsaveconf(void)
 	            "# Created by " PACKAGE_STRING "\n\n");
 
 	for (i = 0; i < arrlen(confoptions); ++i) {
-		fprintf(fs, "%-20s", confoptions[i].name);
+		fprintf(fs, "%-25s ", confoptions[i].name);
 		switch (confoptions[i].type) {
 		case CONF_BOOL:
 			fprintf(fs, "%d", *confoptions[i].value.b);
@@ -257,262 +267,41 @@ void swsaveconf(void)
 	fclose(fs);
 }
 
-struct menuitem {
-	char *config_name;
-	char *description;
+static const struct menuitem keys_menu_items[] = {
+	CONFIG_OPTION("Accelerate",       "key_accelerate"),
+	CONFIG_OPTION("Decelerate",       "key_decelerate"),
+	CONFIG_OPTION("Pull up",          "key_pullup"),
+	CONFIG_OPTION("Pull down",        "key_pulldown"),
+	CONFIG_OPTION("Flip",             "key_flip"),
+	CONFIG_OPTION("Fire machine gun", "key_fire"),
+	CONFIG_OPTION("Drop bomb",        "key_dropbomb"),
+	CONFIG_OPTION("Navigate home",    "key_home"),
+	{0,   NULL},
 };
 
-static const char menukeys[] = "1234567890ABCDEFGHIJKL";
-
-static void ChangeKeyBinding(const struct menuitem *item)
-{
-	const struct conf_option *opt;
-	int key;
-
-	Vid_ClearBuf();
-
-	swcolor(3);
-	swposcur(10, 5);
-	swputs("Press the new key for: ");
-
-	swcolor(2);
-	swposcur(14, 7);
-	swputs(item->description);
-
-	swcolor(1);
-	swposcur(1, 22);
-	swputs("   ESC - Cancel");
-
-	Vid_Update();
-
-	while ((key = Vid_GetKey()) == 0) {
-		Timer_Sleep(50);
-
-		swsndupdate();
-		if (ctlbreak()) {
-			return;
-		}
-	}
-
-	if (!strcasecmp(Vid_KeyName(key), "Escape")) {
-		return;
-	}
-
-	opt = ConfOptionByName(item->config_name);
-	if (opt == NULL) {
-		return;
-	}
-	*opt->value.i = key;
-}
-
-static void DrawMenu(const char *title, const struct menuitem *menu)
-{
-	int i, y, keynum, said_key = 0;
-	int title_len = strlen(title), x = 19 - title_len / 2;
-	Vid_ClearBuf();
-
-	// Planes on the menus look decorative but serve a real
-	// purpose: they give you an impression of what the game will
-	// look like under the different palettes.
-	Vid_DispSymbol(x * 8 - 32, SCR_HGHT - 10,
-	               &symbol_plane[0].sym[0], OWNER_PLAYER1);
-	Vid_DispSymbol((x + title_len) * 8 + 16, SCR_HGHT - 10,
-	               &symbol_plane[0].sym[6], OWNER_PLAYER2);
-
-	swcolor(2);
-	swposcur(x, 2);
-	swputs(title);
-
-	swcolor(3);
-
-	for (i=0, y=0, keynum=0; menu[i].config_name != NULL; ++i, ++y) {
-		const struct conf_option *opt;
-		char *suffix;
-		char buf[40];
-		int key;
-
-		if (strlen(menu[i].config_name) == 0) {
-			continue;
-		}
-
-		if (menu[i].config_name[0] == '>') {
-			key = menu[i].config_name[1];
-			swcolor(1);
-			suffix = " >>>";
-		} else {
-			key = menukeys[keynum];
-			++keynum;
-			suffix = ":";
-
-			if (!said_key) {
-				swposcur(0, 5+y);
-				swputs("Key:");
-				said_key = 1;
-			}
-		}
-		snprintf(buf, sizeof(buf), "%c - %s%s",
-		         key, menu[i].description, suffix);
-
-		swposcur(5, 5+y);
-		swputs(buf);
-		swcolor(3);
-
-		if (strlen(buf) > 22) {
-			++y;
-		}
-
-		swposcur(28, 5+y);
-		opt = ConfOptionByName(menu[i].config_name);
-		if (opt == NULL) {
-			continue;
-		}
-		switch (opt->type) {
-		case CONF_BOOL:
-			swputs(*opt->value.b ? "on" : "off");
-			break;
-		case CONF_INT:
-			if(!strcasecmp(opt->name, "conf_video_palette")) {
-				swputs(Vid_GetVideoPaletteName(*opt->value.i));
-			}
-			break;
-		case CONF_KEY:
-			swputs(Vid_KeyName(*opt->value.i));
-			break;
-		default:
-			break;
-		}
-	}
-
-	swcolor(1);
-
-	swposcur(1, 22);
-	swputs("   ESC - Exit Menu");
-
-	Vid_Update();
-}
-
-static const struct menuitem *MenuItemForKey(const struct menuitem *menu,
-                                             int key)
-{
-	int i, keynum;
-
-	for (i=0, keynum=0; menu[i].config_name != NULL; ++i) {
-		int itemkey;
-		if (strlen(menu[i].config_name) == 0) {
-			continue;
-		} else if (menu[i].config_name[0] == '>') {
-			itemkey = menu[i].config_name[1];
-		} else {
-			itemkey = menukeys[keynum];
-			++keynum;
-		}
-		if (key == itemkey) {
-			return &menu[i];
-		}
-	}
-
-	return NULL;
-}
-
-// Present the given menu to the user. Returns zero if escape was pushed
-// to exit the menu, or if a >jump item was chosen, it returns the key
-// binding associated with it.
-static int RunMenu(const char *title, const struct menuitem *menu)
-{
-	const struct menuitem *pressed;
-	const struct conf_option *opt;
-	int key;
-
-	for (;;) {
-		DrawMenu(title, menu);
-
-		if (ctlbreak()) {
-			swend(NULL, false);
-		}
-
-		key = toupper(swgetc() & 0xff);
-		if (key == 27) {
-			return 0;
-		}
-
-		// check if a number has been pressed for a menu option
-		pressed = MenuItemForKey(menu, key);
-		if (pressed == NULL) {
-			continue;
-		}
-
-		if (pressed->config_name[0] == '>') {
-			return pressed->config_name[1];
-		}
-
-		opt = ConfOptionByName(pressed->config_name);
-		if (opt == NULL) {
-			continue;
-		}
-		switch (opt->type) {
-		case CONF_BOOL:
-			*opt->value.b = !*opt->value.b;
-			break;
-		case CONF_INT:
-			if(!strcasecmp(opt->name, "conf_video_palette")) {
-				*opt->value.i = (*opt->value.i + 1) % Vid_GetNumVideoPalettes();
-				Vid_SetVideoPalette(*opt->value.i);
-			}
-			break;
-		case CONF_KEY:
-			ChangeKeyBinding(pressed);
-			break;
-		default:
-			break;
-		}
-
-		// reset the screen if we need to
-		if (opt->value.b == &vid_fullscreen) {
-			Vid_Reset();
-		}
-
-		swsaveconf();
-	}
-}
-
-static const struct menuitem keys_menu[] = {
-	{"key_accelerate", "Accelerate"},
-	{"key_decelerate", "Decelerate"},
-	{"key_pullup",     "Pull up"},
-	{"key_pulldown",   "Pull down"},
-	{"key_flip",       "Flip"},
-	{"key_fire",       "Fire machine gun"},
-	{"key_dropbomb",   "Drop bomb"},
-	{"key_home",       "Navigate home"},
-	{NULL},
+static const struct menu keys_menu = {
+	FullscreenBackground, "OPTIONS",
+	keys_menu_items,
 };
 
-static const struct menuitem options_menu[] = {
+static const struct menuitem options_menu_items[] = {
 #ifndef NO_FULLSCREEN
-	{"vid_fullscreen",      "Run fullscreen"},
+	CONFIG_OPTION("Run fullscreen",          "vid_fullscreen"),
 #endif
-	{"conf_video_palette",  "Video palette"},
-	{"conf_solidground",    "Solid ground"},
-	{"conf_hudsplats",      "HUD splats"},
-	{"conf_wounded",        "Wounded planes"},
-	{"conf_animals",        "Oxen and birds"},
-	{"conf_big_explosions", "Big oil tank explosions"},
-	{"conf_medals",         "Medals"},
-	{"conf_harrykeys",      "Harry keys mode"},
-	{"",                    ""},
-	{">K",                  "Key bindings"},
-	{NULL},
+	CONFIG_OPTION("Video palette",           "conf_video_palette"),
+	CONFIG_OPTION("Solid ground",            "conf_solidground"),
+	CONFIG_OPTION("HUD splats",              "conf_hudsplats"),
+	CONFIG_OPTION("Wounded planes",          "conf_wounded"),
+	CONFIG_OPTION("Oxen and birds",          "conf_animals"),
+	CONFIG_OPTION("Big oil tank explosions", "conf_big_explosions"),
+	CONFIG_OPTION("Medals",                  "conf_medals"),
+	CONFIG_OPTION("Harry keys mode",         "conf_harrykeys"),
+	{0,   ""},
+	{'K', "Key bindings >>>", SubMenu, &keys_menu},
+	{0,   NULL},
 };
 
-void setconfig(void)
-{
-	for (;;) {
-		switch (RunMenu("OPTIONS", options_menu)) {
-			case 0:
-				return;
-			case 'K':
-				RunMenu("OPTIONS > KEY BINDINGS", keys_menu);
-				break;
-		}
-	}
-}
+const struct menu options_menu = {
+	FullscreenBackground, "OPTIONS > KEY BINDINGS",
+	options_menu_items,
+};
