@@ -17,7 +17,7 @@
 extern char *vidram;
 
 extern void swdisp(void);
-extern void swground(void);
+extern void swground(GRNDTYPE *gptr, int x);
 extern void swputsym(int x, int y, OBJECTS *ob);
 //extern char    fill[];
 
@@ -25,4 +25,6 @@ extern void swinitgrph(void);
 
 extern void colorscreen(int color);
 
+void Notification(const char *s, ...);
+
 #endif
