@@ -17,14 +17,15 @@
 #include <time.h>
 #include <assert.h>
 
+#include "hiscore.h"
 #include "pcsound.h"
 #include "timer.h"
 #include "video.h"
 
 #include "sw.h"
 #include "swasynio.h"
+#include "swcollsn.h"
 #include "swconf.h"
-#include "swdisp.h"
 #include "swgrpha.h"
 #include "swinit.h"
 #include "swgames.h"
@@ -36,7 +37,8 @@
 #include "swsymbol.h"
 #include "swtitle.h"
 
-static const original_ob_t *orig_planes[MAX_PLYR * 2];
+static const original_ob_t *orig_planes[MAX_PLANES];
+static int num_orig_planes;
 GRNDTYPE *ground;
 
 static bool have_savescore = false;
@@ -224,15 +226,15 @@ static void GetEndLevel(OBJECTS *ob)
 
 // plane
 
-OBJECTS *initpln(OBJECTS * obp)
+OBJECTS *initpln(OBJECTS *obp, const original_ob_t *orig_ob)
 {
 	OBJECTS *ob;
 	int x, height, minx, maxx;
 
 	if (!obp) {
-		assert(num_planes < MAX_PLYR);
+		assert(num_planes < MAX_PLANES);
 		ob = allocobj();
-		ob->ob_original_ob = orig_planes[num_planes];
+		ob->ob_original_ob = orig_ob;
 		planes[num_planes] = ob;
 		++num_planes;
 	} else {
@@ -253,9 +255,7 @@ OBJECTS *initpln(OBJECTS * obp)
 	maxx = ob->ob_x + 20;
 	height = 0;
 	for (x = minx; x <= maxx; ++x) {
-		if (ground[x] > height) {
-			height = ground[x];
-		}
+		height = imax(height, ground[x]);
 	}
 	ob->ob_y = height + 13;
 	ob->ob_orig_y = ob->ob_y;
@@ -269,7 +269,7 @@ OBJECTS *initpln(OBJECTS * obp)
 	ob->ob_target = ob->ob_missiletarget = NULL;
 	ob->ob_firing = ob->ob_mfiring = NULL;
 	ob->ob_bombing = ob->ob_bfiring = ob->ob_home = false;
-	ob->ob_newsym = &symbol_plane[0].sym[ob->ob_orient ? 4 : 0];
+	ob->ob_symbol = &symbol_plane[0].sym[ob->ob_orient ? 4 : 0];
 	ob->ob_athome = true;
 	ob->ob_onmap = true;
 	ob->ob_flightscore.combatwound = false;
@@ -299,16 +299,15 @@ OBJECTS *initpln(OBJECTS * obp)
 
 // player
 
-void initplyr(OBJECTS * obp)
+void initplyr(OBJECTS *obp, const original_ob_t *orig_ob)
 {
 	OBJECTS *ob;
 
-	ob = initpln(obp);
+	ob = initpln(obp, orig_ob);
 	if (!obp) {
-		ob->ob_drawf = dispplyr;
 		ob->ob_movef = moveplyr;
-		ob->ob_clr = ob->ob_original_ob->owner;
-		ob->ob_owner = ob;
+		ob->ob_faction = ob->ob_original_ob->faction;
+		ob->ob_clr = ob->ob_faction;
 		endcount = 0;
 
 		ob->ob_plrnum = num_players;
@@ -316,7 +315,6 @@ void initplyr(OBJECTS * obp)
 
 		/* todo: save pointers to all player planes
 		 * and turn consoleplayer into a macro */
-
 		if (ob->ob_plrnum == player) {
 			consoleplayer = ob;
 		}
@@ -325,16 +323,17 @@ void initplyr(OBJECTS * obp)
 
 // computer opponent
 
-void initcomp(OBJECTS * obp)
+void initcomp(OBJECTS *obp, const original_ob_t *orig_ob)
 {
 	OBJECTS *ob;
 
-	ob = initpln(obp);
+	ob = initpln(obp, orig_ob);
 	if (!obp) {
-		ob->ob_drawf = dispcomp;
 		ob->ob_movef = movecomp;
-		ob->ob_clr = 2;
-		ob->ob_owner = planes[1];
+		// TODO: Allow multiple computer-controlled planes belonging
+		// to different factions.
+		ob->ob_faction = FACTION_PLAYER2;
+		ob->ob_clr = ob->ob_faction;
 	}
 	if (playmode == PLAYMODE_SINGLE || playmode == PLAYMODE_NOVICE) {
 		ob->ob_state = FINISHED;
@@ -364,8 +363,46 @@ static int isrange(int x, int y, int ax, int ay)
 	return (7 * dx + 4 * dy) / 8;
 }
 
-// bullet
+// When targets fire at the player, it is sometimes possible that the bullet's
+// first position is inside the target that fired it. This usually doesn't
+// happen because there is space at the top of the target symbols, but in the
+// DOS Network/Author's Edition versions, hangars can actually blow themselves
+// up by shooting their own flag. When spawning a bullet we therefore perform a
+// check to make sure this isn't going to happen. If we detect a collision, we
+// adjust the bullet spawn position very slightly to avoid it.
+static void AdjustBullet(OBJECTS *bullet, OBJECTS *src)
+{
+	OBJECTS next_bullet, next_src;
+	int x, y;
 
+	copyobj(&next_src, src);
+	movexy(&next_src, &x, &y);
+
+	// We keep adjusting until we don't detect a collision.
+	for (;;) {
+		// Where will the bullet be on the next frame?
+		copyobj(&next_bullet, bullet);
+		movexy(&next_bullet, &x, &y);
+
+		if (!CollisionTest(&next_bullet, &next_src)) {
+			return;
+		}
+
+		if (bullet->ob_dx < 0) {
+			bullet->ob_x -= 2;
+		} else {
+			bullet->ob_x += 2;
+		}
+
+		if (bullet->ob_dy < 0) {
+			bullet->ob_y -= 2;
+		} else {
+			bullet->ob_y += 2;
+		}
+	}
+}
+
+// bullet
 void initshot(OBJECTS *obo, OBJECTS * targ)
 {
 	OBJECTS *ob;
@@ -408,25 +445,31 @@ void initshot(OBJECTS *obo, OBJECTS * targ)
 	}
 
 	ob->ob_type = SHOT;
-	ob->ob_x = obo->ob_x + SYM_WDTH / 2;
-	ob->ob_y = obo->ob_y - SYM_HGHT / 2;
+	ob->ob_x = obo->ob_x + obo->ob_symbol->w / 2;
+	ob->ob_y = obo->ob_y - obo->ob_symbol->h / 2;
 	ob->ob_lx = obo->ob_lx;
 	ob->ob_ly = obo->ob_ly;
 
 	ob->ob_life = BULLIFE;
 	ob->ob_owner = obo;
 	ob->ob_clr = obo->ob_clr;
-	ob->ob_newsym = &symbol_pixel;
-	ob->ob_drawf = NULL;
+	ob->ob_symbol = &symbol_pixel;
+	ob->ob_soundf = NULL;
 	ob->ob_movef = moveshot;
 	ob->ob_speed = 0;
 
-	insertx(ob, obo);
+	// For observation balloons, we adjust the position very slightly,
+	// so that the bullets appear to be being fired from the basket.
+	if (obo->ob_type == BALLOON) {
+		ob->ob_y -= 4;
+	}
 
+	AdjustBullet(ob, obo);
+
+	insertx(ob, obo);
 }
 
 // bomb
-
 void initbomb(OBJECTS *obo)
 {
 	OBJECTS *ob;
@@ -466,16 +509,13 @@ void initbomb(OBJECTS *obo)
 	ob->ob_life = BOMBLIFE;
 	ob->ob_owner = obo;
 	ob->ob_clr = obo->ob_clr;
-	ob->ob_newsym = &symbol_bomb[0].sym[0];
-	ob->ob_drawf = dispbomb;
+	ob->ob_symbol = &symbol_bomb[0].sym[0];
 	ob->ob_movef = movebomb;
 
 	insertx(ob, obo);
-
 }
 
 // missile
-
 void initmiss(OBJECTS *obo)
 {
 	OBJECTS *ob;
@@ -509,19 +549,17 @@ void initmiss(OBJECTS *obo)
 	ob->ob_life = MISSLIFE;
 	ob->ob_owner = obo;
 	ob->ob_clr = obo->ob_clr;
-	ob->ob_newsym = &symbol_missile[0].sym[0];
-	ob->ob_drawf = NULL;
+	ob->ob_symbol = &symbol_missile[0].sym[0];
+	ob->ob_soundf = NULL;
 	ob->ob_movef = movemiss;
 	ob->ob_missiletarget = obo->ob_mfiring;
 	ob->ob_orient = ob->ob_accel = ob->ob_flaps = 0;
 	ob->ob_onmap = true;
 
 	insertx(ob, obo);
-
 }
 
 // starburst
-
 void initburst(OBJECTS *obo)
 {
 	OBJECTS *ob;
@@ -562,16 +600,49 @@ void initburst(OBJECTS *obo)
 	ob->ob_life = BURSTLIFE;
 	ob->ob_owner = obo;
 	ob->ob_clr = obo->ob_clr;
-	ob->ob_newsym = &symbol_burst[0].sym[0];
-	ob->ob_drawf = NULL;
+	ob->ob_symbol = &symbol_burst[0].sym[0];
+	ob->ob_soundf = NULL;
 	ob->ob_movef = moveburst;
 
 	insertx(ob, obo);
+}
 
+static void AddPlayerTarget(OBJECTS *ob, const original_ob_t *orig_ob)
+{
+	// TODO: For the time being, player 3 is just interpreted as a
+	// special player number that either maps to player 1 or 2
+	// depending on the game mode; other players are just synonyms
+	// for players 1 and 2.
+	// This may change in the future if we support multiplayer with
+	// more than two players.
+	switch (orig_ob->faction) {
+		case NUM_FACTIONS:
+		case FACTION_NONE:
+			ob->ob_faction = FACTION_NONE;
+			break;
+		case FACTION_PLAYER1:
+		case FACTION_PLAYER5:
+		case FACTION_PLAYER7:
+			ob->ob_faction = FACTION_PLAYER1;
+			break;
+		case FACTION_PLAYER2:
+		case FACTION_PLAYER4:
+		case FACTION_PLAYER6:
+		case FACTION_PLAYER8:
+			ob->ob_faction = FACTION_PLAYER2;
+			break;
+		case FACTION_PLAYER3:
+			if (playmode == PLAYMODE_ASYNCH) {
+				ob->ob_faction = FACTION_PLAYER1;
+			} else {
+				ob->ob_faction = FACTION_PLAYER2;
+			}
+			break;
+	}
+	++numtarg[ob->ob_faction];
 }
 
 // building/target
-
 static OBJECTS *inittarget(const original_ob_t *orig_ob)
 {
 	OBJECTS *ob;
@@ -579,23 +650,22 @@ static OBJECTS *inittarget(const original_ob_t *orig_ob)
 	int minh, maxh, aveh, minx, maxx;
 
 	ob = allocobj();
+
+	// Initial symbol:
+	ob->ob_symbol = &symbol_targets[orig_ob->orient].sym[0];
+
 	minx = ob->ob_x = orig_ob->x;
-	maxx = ob->ob_x + 15;
+	maxx = ob->ob_x + ob->ob_symbol->w - 1;
 	minh = 999;
 	maxh = 0;
 	for (x = minx; x <= maxx; ++x) {
-		if (ground[x] > maxh) {
-			maxh = ground[x];
-		}
-		if (ground[x] < minh) {
-			minh = ground[x];
-		}
+		minh = imin(minh, ground[x]);
+		maxh = imax(maxh, ground[x]);
 	}
-	aveh = (minh + maxh) / 2;
 
-	while ((ob->ob_y = aveh + 16) >= MAX_Y) {
-		--aveh;
-	}
+	aveh = (minh + maxh) / 2;
+	aveh = clamp_max(aveh, MAX_Y - ob->ob_symbol->h - 1);
+	ob->ob_y = aveh + ob->ob_symbol->h;
 
 	for (x = minx; x <= maxx; ++x) {
 		ground[x] = aveh;
@@ -606,41 +676,8 @@ static OBJECTS *inittarget(const original_ob_t *orig_ob)
 	ob->ob_type = TARGET;
 	ob->ob_state = STANDING;
 	ob->ob_orient = orig_ob->orient;
-
-	// TODO: For the time being, player 3 is just interpreted as a
-	// special player number that either maps to player 1 or 2
-	// depending on the game mode; other players are just synonyms
-	// for players 1 and 2.
-	// This may change in the future if we support multiplayer with
-	// more than two players.
-	switch (orig_ob->owner) {
-		case OWNER_NONE:
-		case OWNER_PLAYER1:
-		case OWNER_PLAYER5:
-		case OWNER_PLAYER7:
-			ob->ob_owner = planes[0];
-			++numtarg[0];
-			break;
-		case OWNER_PLAYER2:
-		case OWNER_PLAYER4:
-		case OWNER_PLAYER6:
-		case OWNER_PLAYER8:
-			ob->ob_owner = planes[1];
-			++numtarg[1];
-			break;
-		case OWNER_PLAYER3:
-			if (playmode == PLAYMODE_ASYNCH) {
-				ob->ob_owner = planes[0];
-				++numtarg[0];
-			} else {
-				ob->ob_owner = planes[1];
-				++numtarg[1];
-			}
-			break;
-	}
-	ob->ob_clr = ob->ob_owner->ob_clr;
-	ob->ob_newsym = &symbol_targets[0].sym[0];
-	ob->ob_drawf = disptarg;
+	AddPlayerTarget(ob, orig_ob);
+	ob->ob_clr = ob->ob_faction;
 	ob->ob_movef = movetarg;
 	ob->ob_onmap = true;
 
@@ -661,17 +698,22 @@ static int TargetExplosionSize(int target_type)
 		}
 	}
 
-	if (num_pixels > 64) {
-		return 64;
-	} else if (num_pixels < 10) {
-		return 10;
-	} else {
-		return num_pixels;
-	}
+	return clamp_range(10, num_pixels, 64);
+}
+
+// The original Sopwith sprites were all 16x16. Since we now support larger
+// sprites, we apply an adjustment so that entire targets appear to be
+// exploding. This is no-op when the symbol is 16x16 like the original.
+static void ApplySpriteSizeOffset(OBJECTS *ob, OBJECTS *obo, int angle)
+{
+	int rangex = clamp_min(obo->ob_symbol->w, 16) - 16,
+	    rangey = clamp_min(obo->ob_symbol->h, 16) - 16;
+	ob->ob_x += COS(angle) * rangex / (256 * 2);
+	ob->ob_y += SIN(angle) * rangey / (256 * 2);
+	ob->ob_y -= rangey;
 }
 
 // explosion
-
 void initexpl(OBJECTS *obo, int small)
 {
 	OBJECTS *ob;
@@ -682,8 +724,8 @@ void initexpl(OBJECTS *obo, int small)
 	bool mansym;
 	int orient;
 
-	obox = obo->ob_x + (obo->ob_newsym->w / 2);
-	oboy = obo->ob_y + (obo->ob_newsym->h / 2);
+	obox = obo->ob_x + (obo->ob_symbol->w / 2);
+	oboy = obo->ob_y + (obo->ob_symbol->h / 2);
 	obodx = obo->ob_dx >> 2;
 	obody = obo->ob_dy >> 2;
 	oboclr = obo->ob_clr;
@@ -738,6 +780,8 @@ void initexpl(OBJECTS *obo, int small)
 
 		ob->ob_x = obox + ob->ob_dx;
 		ob->ob_y = oboy + ob->ob_dy;
+		ApplySpriteSizeOffset(ob, obo, i);
+
 		explseed *= ob->ob_x * ob->ob_y;
 		explseed += 7491;
 		if (!explseed) {
@@ -747,7 +791,8 @@ void initexpl(OBJECTS *obo, int small)
 		ob->ob_life = EXPLLIFE;
 		orient = ob->ob_orient = (explseed & 0x01C0) >> 6;
 		if (mansym && (!orient || orient == 7)) {
-			mansym = orient = ob->ob_orient = 0;
+			orient = ob->ob_orient = 0;
+			mansym = false;
 			ob->ob_dx = obodx;
 			ob->ob_dy = -gminspeed;
 		}
@@ -755,8 +800,7 @@ void initexpl(OBJECTS *obo, int small)
 		ob->ob_lx = ob->ob_ly = ob->ob_hitcount = ob->ob_speed = 0;
 		ob->ob_owner = obo;
 		ob->ob_clr = oboclr;
-		ob->ob_newsym = &symbol_debris[0].sym[0];
-		ob->ob_drawf = dispexpl;
+		ob->ob_symbol = &symbol_debris[0].sym[0];
 		ob->ob_movef = moveexpl;
 
 		if (orient) {
@@ -768,7 +812,6 @@ void initexpl(OBJECTS *obo, int small)
 }
 
 // smoke from falling plane
-
 void initsmok(OBJECTS *obo)
 {
 	OBJECTS *ob;
@@ -787,13 +830,12 @@ void initsmok(OBJECTS *obo)
 	ob->ob_lx = ob->ob_ly = ob->ob_ldx = ob->ob_ldy = 0;
 	ob->ob_life = SMOKELIFE;
 	ob->ob_owner = obo;
-	ob->ob_drawf = NULL;
+	ob->ob_soundf = NULL;
 	ob->ob_movef = movesmok;
 	ob->ob_clr = obo->ob_clr;
 }
 
 // birds
-
 static OBJECTS *initflock(const original_ob_t *orig_ob)
 {
 	OBJECTS *ob;
@@ -816,9 +858,9 @@ static OBJECTS *initflock(const original_ob_t *orig_ob)
 	ob->ob_dy = ob->ob_lx = ob->ob_ly = ob->ob_ldx = ob->ob_ldy = 0;
 	ob->ob_orient = 0;
 	ob->ob_life = FLOCKLIFE;
-	ob->ob_owner = ob;
-	ob->ob_newsym = &symbol_flock[0].sym[0];
-	ob->ob_drawf = NULL;
+	ob->ob_faction = FACTION_NONE;
+	ob->ob_symbol = &symbol_flock[0].sym[0];
+	ob->ob_soundf = NULL;
 	ob->ob_movef = moveflck;
 	ob->ob_clr = 1;
 	ob->ob_onmap = true;
@@ -831,7 +873,6 @@ static OBJECTS *initflock(const original_ob_t *orig_ob)
 }
 
 // single bird
-
 void initbird(OBJECTS *obo, int i)
 {
 	OBJECTS *ob;
@@ -854,16 +895,43 @@ void initbird(OBJECTS *obo, int i)
 	ob->ob_orient = ob->ob_lx = ob->ob_ly = ob->ob_ldx = ob->ob_ldy =
 	    0;
 	ob->ob_life = BIRDLIFE;
-	ob->ob_owner = obo;
-	ob->ob_newsym = &symbol_bird[0].sym[0];
-	ob->ob_drawf = NULL;
+	ob->ob_faction = obo->ob_faction;
+	ob->ob_symbol = &symbol_bird[0].sym[0];
+	ob->ob_soundf = NULL;
 	ob->ob_movef = movebird;
 	ob->ob_clr = obo->ob_clr;
 	insertx(ob, &topobj);
 }
 
-// oxen
+static OBJECTS *initballoon(const original_ob_t *orig_ob)
+{
+	OBJECTS *ob;
 
+	ob = allocobj();
+	if (!ob) {
+		return NULL;
+	}
+
+	ob->ob_type = BALLOON;
+	ob->ob_state = FLYING;
+	ob->ob_life = 1;
+	ob->ob_x = orig_ob->x;
+	ob->ob_y = MAX_Y - 16 + SIN(orig_ob->x) / 32;
+	ob->ob_dx = 0;
+	ob->ob_dy = 0;
+	ob->ob_orient = 0;
+	ob->ob_symbol = &symbol_balloon[0].sym[0];
+	ob->ob_soundf = NULL;
+	ob->ob_movef = moveballoon;
+	ob->ob_faction = orig_ob->faction;
+	ob->ob_clr = ob->ob_faction;
+	ob->ob_onmap = true;
+	AddPlayerTarget(ob, orig_ob);
+
+	return ob;
+}
+
+// oxen
 static OBJECTS *initox(const original_ob_t *orig_ob)
 {
 	OBJECTS *ob;
@@ -883,9 +951,9 @@ static OBJECTS *initox(const original_ob_t *orig_ob)
 	ob->ob_y = ground[ob->ob_x] + 16;
 	ob->ob_orient = ob->ob_lx = ob->ob_ly = ob->ob_ldx =
 	    ob->ob_ldy = ob->ob_dx = ob->ob_dy = 0;
-	ob->ob_owner = ob;
-	ob->ob_newsym = &symbol_ox[0].sym[orig_ob->transform];
-	ob->ob_drawf = NULL;
+	ob->ob_faction = FACTION_NONE;
+	ob->ob_symbol = &symbol_ox[0].sym[orig_ob->transform];
+	ob->ob_soundf = NULL;
 	ob->ob_movef = moveox;
 	ob->ob_clr = 1;
 	return ob;
@@ -897,8 +965,7 @@ static void inittargets(void)
 	const original_ob_t *orig_ob;
 	int i;
 
-	numtarg[0] = 0;
-	numtarg[1] = 0;
+	memset(numtarg, 0, sizeof(numtarg));
 
 	for (i = 0; i < currgame->gm_num_objects; i++) {
 		orig_ob = &currgame->gm_objects[i];
@@ -912,6 +979,9 @@ static void inittargets(void)
 			case FLOCK:
 				ob = initflock(orig_ob);
 				break;
+			case BALLOON:
+				ob = initballoon(orig_ob);
+				break;
 			default:
 				continue;
 		}
@@ -935,8 +1005,22 @@ static void initgdep(void)
 	targrnge *= targrnge;
 }
 
+OBJECTS *PlaneForFaction(faction_t f)
+{
+	int i;
+
+	for (i = 0; i < num_planes; i++) {
+		if (planes[i]->ob_faction == f) {
+			return planes[i];;
+		}
+	}
+
+	return NULL;
+}
+
 void swinitlevel(void)
 {
+	original_ob_t *player1_ob, *player2_ob;
 	int i;
 
 	if (have_custom_level) {
@@ -945,9 +1029,10 @@ void swinitlevel(void)
 		currgame = &original_level;
 	}
 
+	Vid_ShowTouchGameControls();
+
 	// clear out any waiting keys. this stops, eg. 's' on the
 	// menu from toggling sound once the game starts
-
 	Vid_GetGameKeys();
 
 	if (playmode == PLAYMODE_ASYNCH) {
@@ -962,37 +1047,59 @@ void swinitlevel(void)
 
 	num_players = 0;
 	num_planes = 0;
+	num_orig_planes = 0;
 	memset(planes, 0, sizeof(planes));
 	memset(orig_planes, 0, sizeof(orig_planes));
 
+	player1_ob = NULL;
+	player2_ob = NULL;
 	for (i = 0; i < currgame->gm_num_objects; i++) {
-		int pln_index;
-		if (currgame->gm_objects[i].type != PLANE) {
+		original_ob_t *plane = &currgame->gm_objects[i];
+		if (plane->type != PLANE) {
 			continue;
 		}
-		pln_index = currgame->gm_objects[i].owner - 1;
-		if (pln_index >= 0 && pln_index < MAX_PLYR * 2) {
-			orig_planes[pln_index] = &currgame->gm_objects[i];
+		if (playmode == PLAYMODE_ASYNCH
+		 && plane->faction > FACTION_PLAYER2) {
+			continue;
+		}
+		if (plane->faction > FACTION_PLAYER4) {
+			// Player 5-8 planes currently unused
+			continue;
+		}
+		orig_planes[num_orig_planes] = plane;
+		++num_orig_planes;
+		if (player1_ob == NULL && plane->faction == FACTION_PLAYER1) {
+			player1_ob = plane;
+		}
+		if (player2_ob == NULL && plane->faction == FACTION_PLAYER2) {
+			player2_ob = plane;
 		}
 	}
+	assert(player1_ob != NULL);
 
 	if (keydelay == -1) {
 		keydelay = 1;
 	}
 
+	initplyr(NULL, player1_ob);
+
 	if (playmode == PLAYMODE_ASYNCH) {
 		maxcrash = MAXCRASH * 2;
-		init2asy();
+		assert(player2_ob != NULL);
+		initplyr(NULL, player2_ob);
 	} else {
 		maxcrash = MAXCRASH;
+		player2_ob = NULL;
+	}
 
-		// single player. we spawn as many enemy planes as we have
-		// defined within the map.
-		// TODO: We should have a way to define the number of enemy
-		// planes in levels.
-		initplyr(NULL);
-		for (i = 1; i < MAX_PLYR && orig_planes[i] != NULL; i++) {
-			initcomp(NULL);
+	// single player. we spawn as many enemy planes as we have
+	// defined within the map.
+	// TODO: We should have a way to define the number of enemy
+	// planes in levels.
+	for (i = 0; i < num_orig_planes; i++) {
+		if (orig_planes[i] != player1_ob
+		 && orig_planes[i] != player2_ob) {
+			initcomp(NULL, orig_planes[i]);
 		}
 	}
 
@@ -1014,12 +1121,11 @@ void swinitlevel(void)
 
 void swrestart(void)
 {
-	OBJECTS *ob;
+	OBJECTS *ob = planes[player];
 	int inc;
 	int time;
-		
+
 	if (consoleplayer->ob_endsts == WINNER) {
-		ob = planes[player];
 		inc = 0;
 
 		GetEndLevel(ob);
@@ -1041,13 +1147,14 @@ void swrestart(void)
 		savescore = ob->ob_score;
 		have_savescore = true;
 	} else {
+		NewHighScore(&ob->ob_score);
+
 		// gamenum = 0;
 		// allow variable start level -- Jesse
 		gamenum = starting_level;
 		have_savescore = false;
 
 		// sh 28/10/2001: go back to the title screen
-
 		playmode = PLAYMODE_UNSET;
 	}
 
@@ -1111,7 +1218,7 @@ void swinit(int argc, char *argv[])
 			a = true;
 			asynmode = ASYN_CONNECT;
 			++i;
-			snprintf(asynhost, sizeof(asynhost), "%s", argv[i]);
+			asynhost = argv[i];
 		} else if (!strcmp(argv[i], "-p")) {
 			if (i + 1 >= argc) {
 				ErrorExit("insufficient arguments to -p");
@@ -1139,6 +1246,7 @@ void swinit(int argc, char *argv[])
 	Timer_Init();
 	Vid_Init();
 	Vid_SetVideoPalette(conf_video_palette);
+	InitHighScores();
 
 	// Don't init speaker if started with -q (quiet)
 	if (soundflg) {
