@@ -2,12 +2,11 @@
 // Copyright(C) 1984-2000 David L. Clark
 // Copyright(C) 2001-2005 Simon Howard
 //
-// This program is free software; you can redistribute it and/or modify it
-// under the terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2 of the License, or (at your
-// option) any later version. This program is distributed in the hope that
-// it will be useful, but WITHOUT ANY WARRANTY; without even the implied
-// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// You can redistribute and/or modify this program under the terms of the
+// GNU General Public License version 2 as published by the Free Software
+// Foundation, or any later version. This program is distributed WITHOUT
+// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.
 //
 
 #ifndef __SWSOUND_H__
@@ -28,9 +27,12 @@ extern void sound(int type, int parm, OBJECTS *ob);
 extern void swsound(void);
 extern void initsound(OBJECTS *obp, int type);
 extern void stopsound(OBJECTS *ob);
+extern void soundoff(void);
+extern void swsndupdate(void);
+
 extern OBJECTS *ob;
 extern unsigned freq;
 extern unsigned  modulo;
-extern void swsndupdate(void);
+extern char *expltune;
 
 #endif
