@@ -49,7 +49,7 @@ void swdispsplats(void)
 
 		ob.ob_type = DUMMYTYPE;
 		ob.ob_clr = splats[i].clr;
-		ob.ob_newsym = splats[i].sym;
+		ob.ob_symbol = splats[i].sym;
 
 		swputsym(splats[i].x, splats[i].y, &ob);
 	}
@@ -88,7 +88,7 @@ void swsplatbird(void)
 	splat.x = (unsigned) (seed % (SCR_WDTH - 32));
 	splat.y = (unsigned) (seed % (SCR_HGHT - 60)) + 60;
 	splat.sym = &symbol_birdsplat[0].sym[0];
-	splat.clr = OWNER_PLAYER2;
+	splat.clr = FACTION_PLAYER2;
 
 	AddSplat(&splat);
 }
@@ -102,7 +102,7 @@ void swwindshot(void)
 	splat.x = (unsigned) (seed % (SCR_WDTH - 16));
 	splat.y = (unsigned) (seed % (SCR_HGHT - 50)) + 50;
 	splat.sym = &symbol_shotwin[0].sym[0];
-	splat.clr = OWNER_PLAYER1;
+	splat.clr = FACTION_PLAYER1;
 
 	AddSplat(&splat);
 }
