@@ -127,6 +127,40 @@ static const char *swhitsym[] = {
 	"            - * *   -           \n"
 	"            * * *   -           \n"
 	"            * * *               \n",
+	// -------------------------------
+	"  - - - - * * - - - -           \n"
+	"    - - - * * - - -             \n"
+	"      - - * * - -               \n"
+	"          * *                   \n"
+	"          * *                   \n"
+	"          * *                   \n"
+	"        * * * *                 \n"
+	"        * * * *                 \n"
+	"        * * * *                 \n"
+	"        * * * *                 \n"
+	"- - - - - - - - - - - - - -     \n"
+	"  - - - - - - - - - - - - - - - \n"
+	"- - - - - - - - - - - - - - - - \n"
+	"-   - - - - - - - - - - - - - - \n"
+	"        * * * *                 \n"
+	"        * * * *                 \n",
+	// -------------------------------
+	"        * * * -                 \n"
+	"        * * * - *               \n"
+	"        * * * - * * *           \n"
+	"          * * - * *             \n"
+	"            * - * *             \n"
+	"              * * *             \n"
+	"              * * *             \n"
+	"              * * *             \n"
+	"          -   * * -             \n"
+	"          -   * * -             \n"
+	"          - - - - -   *         \n"
+	"          -   * * - * * *       \n"
+	"          - - - - -   *         \n"
+	"          -   * * -             \n"
+	"          -   * * *             \n"
+	"              * * *             \n",
 };
 
 static const char *swwinsym[] = {  /*  Win plane pixel array  */
@@ -396,6 +430,73 @@ static const char *swtrgsym[] = {
 	custom_target_sym,  // TARGET_CUSTOM_PASSIVE4
 	custom_target_sym,  // TARGET_CUSTOM_PASSIVE5
 
+	// TARGET_RADIO_TOWER:
+	"              #                 \n"
+	"            #   #               \n"
+	"          # * * * #             \n"
+	"          # #     #             \n"
+	"          #   #   #             \n"
+	"          #     # #             \n"
+	"          # * * * #             \n"
+	"          #       #             \n"
+	"        #   #   #   #           \n"
+	"        #     #     #           \n"
+	"        # * * * * * #           \n"
+	"        #   #   #   #           \n"
+	"        # #       # #           \n"
+	"        # * * * * * #           \n"
+	"        #           #           \n"
+	"      #   #       #   #         \n"
+	"      #     #   #     #         \n"
+	"      # * * * * * * * #         \n"
+	"      #     #   #     #         \n"
+	"      #   #       #   #         \n"
+	"      # #           # #         \n"
+	"      # * * * * * * * #         \n"
+	"      #               #         \n"
+	"    #   #           #   #       \n"
+	"    #     #       #     #       \n"
+	"    #       #   #       #       \n"
+	"    # * * * * * * * * * #       \n"
+	"    #       #   #       #       \n"
+	"    #     #       #     #       \n"
+	"    #   #           #   #       \n"
+	"    # #               # #       \n"
+	"    #                   #       \n",
+
+	// TARGET_WATER_TOWER:
+	"          - - - - -             \n"
+	"        - - - - - - -           \n"
+	"        * * * * * * *           \n"
+	"        * * * * * * *           \n"
+	"        * * * * * * *           \n"
+	"        * * * * * * *           \n"
+	"        - - - - - - -           \n"
+	"        * * * * * * *           \n"
+	"          # * * * #             \n"
+	"          #   -   #             \n"
+	"          #   -   #             \n"
+	"        #   # - #   #           \n"
+	"        * * * * * * *           \n"
+	"        #   # - #   #           \n"
+	"        # #   -   # #           \n"
+	"        #     -     #           \n"
+	"      #   #   -   #   #         \n"
+	"      #     # - #     #         \n"
+	"      * * * * * * * * *         \n"
+	"      #     # - #     #         \n"
+	"      #   #   -   #   #         \n"
+	"      # #     -     # #         \n"
+	"      #       -       #         \n"
+	"    #   #     -     #   #       \n"
+	"    #     #   -   #     #       \n"
+	"    #       # - #       #       \n"
+	"    * * * * * * * * * * *       \n"
+	"    #       # - #       #       \n"
+	"    #     #   -   #     #       \n"
+	"    #   #     -     #   #       \n"
+	"    # #       -       # #       \n"
+	"    #         -         #       \n",
 };
 
 static const char destroyed_building[] =
@@ -488,6 +589,42 @@ static const char destroyed_custom[] =
 	"  * * * * * * * * * * * * * * * \n"
 	"  * * * * * * * * * * * * * * * \n";
 
+static const char destroyed_radio_tower[] =
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"      *                         \n"
+	"        *             #         \n"
+	"      *   *     #     #         \n"
+	"    # * #   *   # # # # * *     \n"
+	"  #   * #   # * #     *     #   \n"
+	"  # # *   # #   #   # # # # *   \n";
+
+static const char destroyed_water_tower[] =
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"              *       -         \n"
+	"              * *   * -         \n"
+	"      *         # * * * -       \n"
+	"    # * *   - * # * * # -       \n"
+	"  # *   # * - * #   # *   -     \n"
+	"  # # #   # * - # #   * # -     \n";
+
 // There is one entry in this array for each in swtrgsym, so that different
 // targets can have different "destroyed" symbols:
 static const char *swhtrsym[] = {
@@ -509,6 +646,113 @@ static const char *swhtrsym[] = {
 	destroyed_custom,  // TARGET_CUSTOM_PASSIVE3
 	destroyed_custom,  // TARGET_CUSTOM_PASSIVE4
 	destroyed_custom,  // TARGET_CUSTOM_PASSIVE5
+	destroyed_radio_tower,  // TARGET_RADIO_TOWER
+	destroyed_water_tower,  // TARGET_WATER_TOWER
+};
+
+static const char *swballoonsym[] = {
+	// Spherical balloon, tilt left:
+	"        # # # # # # # #         \n"
+	"      # * * # * # * # * #       \n"
+	"    # * * # * * # * # * * #     \n"
+	"  # * * # * * # * * * # * * #   \n"
+	"  # * * # * * # * * * # * * #   \n"
+	"  # * * # * * # * * * # * * #   \n"
+	"  # * * # * * # * * * # * * #   \n"
+	"  # * * # * * # * * * # * * #   \n"
+	"  # * * # * * # * * * # * * #   \n"
+	"    # * * # * * # * # * * #     \n"
+	"      # * * # * # * # * #       \n"
+	"        # # # # # # # #         \n"
+	"          #         #           \n"
+	"            - - - - -           \n"
+	"            - - - - -           \n"
+	"            - - - - -           \n",
+	// Spherical, center:
+	"        # # # # # # # # #       \n"
+	"      # * * # * # * # * * #     \n"
+	"    # * * # * * # * * # * * #   \n"
+	"    # * * # * * # * * # * * #   \n"
+	"  # * * # * * * # * * * # * * # \n"
+	"  # * * # * * * # * * * # * * # \n"
+	"  # * * # * * * # * * * # * * # \n"
+	"  # * * # * * * # * * * # * * # \n"
+	"    # * * # * * # * * # * * #   \n"
+	"    # * * # * * # * * # * * #   \n"
+	"      # * * # * # * # * * #     \n"
+	"        # # # # # # # # #       \n"
+	"            #       #           \n"
+	"            - - - - -           \n"
+	"            - - - - -           \n"
+	"            - - - - -           \n",
+	// Spherical, tilt right:
+	"          # # # # # # # #       \n"
+	"        # * # * # * # * * #     \n"
+	"      # * * # * # * * # * * #   \n"
+	"    # * * # * * * # * * # * * # \n"
+	"    # * * # * * * # * * # * * # \n"
+	"    # * * # * * * # * * # * * # \n"
+	"    # * * # * * * # * * # * * # \n"
+	"    # * * # * * * # * * # * * # \n"
+	"    # * * # * * * # * * # * * # \n"
+	"      # * * # * # * * # * * #   \n"
+	"        # * # * # * # * * #     \n"
+	"          # # # # # # # #       \n"
+	"            #         #         \n"
+	"            - - - - -           \n"
+	"            - - - - -           \n"
+	"            - - - - -           \n",
+	// Caquot-style observation balloon, tilt left:
+	"                          # #   \n"
+	"                      # # * #   \n"
+	"    # # # # # # # # # * * * #   \n"
+	"  # * * * * * * * * * # # # #   \n"
+	"# * * * * * * * * * * * * * #   \n"
+	"# * * * * * * * * * * * * * #   \n"
+	"# # # # # # # # # # # # # # #   \n"
+	"# * * * * * * * * * * * * * #   \n"
+	"# * * * * * * * * * * * * * #   \n"
+	"  # * * * * * * * * * # # # #   \n"
+	"    # # # # # # # # # * * * #   \n"
+	"        #         #   # # * #   \n"
+	"          #       #       # #   \n"
+	"          - - - - -             \n"
+	"          - - - - -             \n"
+	"          - - - - -             \n",
+	// Caquot, center:
+	"                          # #   \n"
+	"                      # # * * # \n"
+	"      # # # # # # # # * * * * # \n"
+	"  # # * * * * * * * * # # # # # \n"
+	"# * * * * * * * * * * * * * * # \n"
+	"# * * * * * * * * * * * * * * # \n"
+	"# # # # # # # # # # # # # # # # \n"
+	"# * * * * * * * * * * * * * * # \n"
+	"# * * * * * * * * * * * * * * # \n"
+	"  # # * * * * * * * * # # # # # \n"
+	"      # # # # # # # # * * * * # \n"
+	"          #       #   # # * * # \n"
+	"          #       #       # #   \n"
+	"          - - - - -             \n"
+	"          - - - - -             \n"
+	"          - - - - -             \n",
+	// Caquot, tilt right:
+	"                          # # # \n"
+	"                        # * * # \n"
+	"      # # # # # # # # # * * * # \n"
+	"    # * * * * * * * * * # # # # \n"
+	"  # * * * * * * * * * * * * * # \n"
+	"  # * * * * * * * * * * * * * # \n"
+	"  # # # # # # # # # # # # # # # \n"
+	"  # * * * * * * * * * * * * * # \n"
+	"  # * * * * * * * * * * * * * # \n"
+	"    # * * * * * * * * * # # # # \n"
+	"      # # # # # # # # # * * * # \n"
+	"          #         #   # * * # \n"
+	"          #       #       # # # \n"
+	"          - - - - -             \n"
+	"          - - - - -             \n"
+	"          - - - - -             \n",
 };
 
 static const char *swexpsym[] = {
@@ -854,11 +1098,35 @@ static void Rotate(int *x, int *y, int w, int h, int rotations, bool mirror)
 	}
 }
 
-static void SopsymFromText(sopsym_t *sym, const char *text, int w, int h,
-                           int rotations, bool mirror)
+static void GetSymbolDimensions(const char *text, int *w, int *h)
+{
+	const char *p;
+
+	*w = 0;
+	*h = 0;
+
+	for (p = text; *p != '\0';) {
+		const char *line_start = p, *line_end;
+		while (*p != '\0' && *p != '\n') {
+			++p;
+		}
+		line_end = p;
+		if (*p == '\0' && line_start == line_end) {
+			break;
+		}
+		*w = imax(*w, (line_end - line_start + 1) / 2);
+		++*h;
+		++p;
+	}
+}
+
+static void SopsymFromText(sopsym_t *sym, const char *text, int rotations,
+                           bool mirror)
 {
 	const char *p, *p2;
-	int x, y, dx, dy, c;
+	int x, y, dx, dy, c, w, h;
+
+	GetSymbolDimensions(text, &w, &h);
 
 	if ((rotations & 1) == 0) {
 		sym->w = w;
@@ -892,14 +1160,14 @@ static void SopsymFromText(sopsym_t *sym, const char *text, int w, int h,
 	}
 }
 
-void SymsetFromText(symset_t *s, const char *text, int w, int h)
+void SymsetFromText(symset_t *s, const char *text)
 {
 	int r;
 
 	for (r = 0; r < 4; r++)
 	{
-		SopsymFromText(&s->sym[r], text, w, h, r, false);
-		SopsymFromText(&s->sym[r + 4], text, w, h, r, true);
+		SopsymFromText(&s->sym[r], text, r, false);
+		SopsymFromText(&s->sym[r + 4], text, r, true);
 	}
 }
 
@@ -938,10 +1206,11 @@ symset_t symbol_birdsplat[1];            // swsplsym
 symset_t symbol_missile[4];              // swmscsym
 symset_t symbol_burst[2];                // swbstsym
 symset_t symbol_plane[4];                // swplnsym
-symset_t symbol_plane_hit[2];            // swhitsym
+symset_t symbol_plane_hit[4];            // swhitsym
 symset_t symbol_plane_win[4];            // swwinsym
 symset_t symbol_medal[3];                // swmedalsym
 symset_t symbol_ribbon[6];               // swribbonsym
+symset_t symbol_balloon[6];              // swballoonsym
 
 // special symbol for single pixel (bullets etc)
 
@@ -954,32 +1223,33 @@ sopsym_t symbol_pixel = {
 };
 
 // generate array of symset_t structs from array of strings:
-#define SYMSETS_FROM_TEXT(text, w, h, out)                        \
-        { int _i;                                                 \
-          for (_i = 0; _i < arrlen(out); ++_i) {                  \
-             InitSymset(&(out)[_i], #text, _i);                   \
-             SymsetFromText(&(out)[_i], (text)[_i], (w), (h));    \
-          }                                                       \
+#define SYMSETS_FROM_TEXT(text, out)                    \
+        { int _i;                                       \
+          for (_i = 0; _i < arrlen(out); ++_i) {        \
+             InitSymset(&(out)[_i], #text, _i);         \
+             SymsetFromText(&(out)[_i], (text)[_i]);    \
+          }                                             \
         }
 
 void GenerateSymbols(void)
 {
-	SYMSETS_FROM_TEXT(swbmbsym, 8, 8, symbol_bomb);
-	SYMSETS_FROM_TEXT(swtrgsym, 16, 16, symbol_targets);
-	SYMSETS_FROM_TEXT(swexpsym, 8, 8, symbol_debris);
-	SYMSETS_FROM_TEXT(swflksym, 16, 16, symbol_flock);
-	SYMSETS_FROM_TEXT(swbrdsym, 4, 2, symbol_bird);
-	SYMSETS_FROM_TEXT(swoxsym, 16, 16, symbol_ox);
-	SYMSETS_FROM_TEXT(swmscsym, 8, 8, symbol_missile);
-	SYMSETS_FROM_TEXT(swbstsym, 8, 8, symbol_burst);
-	SYMSETS_FROM_TEXT(swplnsym, 16, 16, symbol_plane);
-	SYMSETS_FROM_TEXT(swhitsym, 16, 16, symbol_plane_hit);
-	SYMSETS_FROM_TEXT(swwinsym, 16, 16, symbol_plane_win);
-	SYMSETS_FROM_TEXT(swmedalsym, 8, 12, symbol_medal);
-	SYMSETS_FROM_TEXT(swribbonsym, 8, 2, symbol_ribbon);
-	SYMSETS_FROM_TEXT(swhtrsym, 16, 16, symbol_target_hit);
-	SYMSETS_FROM_TEXT(swshtsym, 16, 16, symbol_shotwin);
-	SYMSETS_FROM_TEXT(swsplsym, 32, 32, symbol_birdsplat);
+	SYMSETS_FROM_TEXT(swbmbsym, symbol_bomb);
+	SYMSETS_FROM_TEXT(swtrgsym, symbol_targets);
+	SYMSETS_FROM_TEXT(swexpsym, symbol_debris);
+	SYMSETS_FROM_TEXT(swflksym, symbol_flock);
+	SYMSETS_FROM_TEXT(swbrdsym, symbol_bird);
+	SYMSETS_FROM_TEXT(swoxsym, symbol_ox);
+	SYMSETS_FROM_TEXT(swmscsym, symbol_missile);
+	SYMSETS_FROM_TEXT(swbstsym, symbol_burst);
+	SYMSETS_FROM_TEXT(swplnsym, symbol_plane);
+	SYMSETS_FROM_TEXT(swhitsym, symbol_plane_hit);
+	SYMSETS_FROM_TEXT(swwinsym, symbol_plane_win);
+	SYMSETS_FROM_TEXT(swmedalsym, symbol_medal);
+	SYMSETS_FROM_TEXT(swribbonsym, symbol_ribbon);
+	SYMSETS_FROM_TEXT(swhtrsym, symbol_target_hit);
+	SYMSETS_FROM_TEXT(swshtsym, symbol_shotwin);
+	SYMSETS_FROM_TEXT(swsplsym, symbol_birdsplat);
+	SYMSETS_FROM_TEXT(swballoonsym, symbol_balloon);
 }
 
 //
