@@ -19,6 +19,8 @@
 
 #include "sw.h"
 
+#define TOUCH_AREA_HEIGHT 160
+
 enum gamekey {
 	KEY_UNKNOWN,
 	KEY_PULLUP,
@@ -35,18 +37,25 @@ enum gamekey {
 	NUM_KEYS,
 };
 
-// which keys are currently down
-// this is actually a simple bitfield
-// bit 0 is whether the button is currently down
-// bit 1 is whether the button has been pressed
-//       since the last call of Vid_GetGameKeys
-// in this way, every button press will have an effect:
-// if it is done based on what is currently down it is
-// possible to miss keypresses (if you press and release
-// a button fast enough)
+enum menukey {
+	MENUKEY_NONE,
+	MENUKEY_START,
+	MENUKEY_BACK,
+	MENUKEY_UP,
+	MENUKEY_DOWN,
+};
+
+// Which keys are currently down or have been pressed since the last call
+// to Vid_GetGameKeys:
+
+#define KEYDOWN_KEYBOARD     (1 << 0)
+#define KEYDOWN_WAS_PRESSED  (1 << 1)
+#define KEYDOWN_TOUCH        (1 << 2)
+#define KEYDOWN_GAMEPAD      (1 << 3)
 
 extern int keysdown[NUM_KEYS];
 extern int keybindings[NUM_KEYS];
+extern int controller_bindings[NUM_KEYS];
 
 extern bool vid_fullscreen;
 extern uint8_t *vid_vram;
@@ -99,8 +108,8 @@ extern void Vid_XorPixel(int x, int y, int clr);
 // draw a symbol
 
 extern void Vid_DispSymbol(int x, int y, sopsym_t *symbol,
-                           ob_owner_t owner);
-extern int Vid_FuselageColor(ob_owner_t clr);
+                           faction_t f);
+extern int Vid_FuselageColor(faction_t f);
 
 // clear screen
 
@@ -110,4 +119,29 @@ char *Vid_GetPrefPath(void);
 
 void ErrorExit(char *s, ...);
 
+enum touch_button_type {
+	TOUCH_BUTTON_END,
+	TOUCH_BUTTON_LABEL,
+	TOUCH_BUTTON_GAME_KEY,
+	TOUCH_BUTTON_KEYPRESS,
+	TOUCH_BUTTON_CLOSE,
+};
+
+struct touch_button {
+	enum touch_button_type type;
+	const char *label;
+	int x, y;
+	int param;
+};
+
+extern void Vid_DrawTouchControls(void);
+extern const struct touch_button *Vid_GetTouchButton(int x, int y);
+extern void Vid_TouchButtonPress(const struct touch_button *b, bool pressed);
+extern void Vid_ShowTouchGameControls(void);
+extern void Vid_ShowTouchKeys(const char *keys);
+
+enum menukey Vid_ControllerMenuKey(void);
+bool Vid_HaveController(void);
+const char *Vid_ControllerButtonName(enum gamekey key);
+
 #endif
